"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeConnection = void 0;
const node_1 = require("vscode-languageserver/node");
const fs = __importStar(require("fs"));
const vscode_languageserver_textdocument_1 = require("vscode-languageserver-textdocument");
const isColor_1 = __importDefault(require("./utils/isColor"));
const protocol_1 = require("./utils/protocol");
const findAll_1 = require("./utils/findAll");
const indexToPosition_1 = require("./utils/indexToPosition");
const getCurrentWord_1 = require("./utils/getCurrentWord");
const isInFunctionExpression_1 = require("./utils/isInFunctionExpression");
const CSSVariableManager_1 = __importStar(require("./CSSVariableManager"));
const makeConnection = () => {
    // Create a connection for the server, using Node's IPC as a transport.
    // Also include all preview / proposed LSP features.
    const connection = (0, node_1.createConnection)(process.stdin, process.stdout);
    // Create a simple text document manager.
    const documents = new node_1.TextDocuments(vscode_languageserver_textdocument_1.TextDocument);
    let hasConfigurationCapability = false;
    let hasWorkspaceFolderCapability = false;
    let hasDiagnosticRelatedInformationCapability = false;
    const cssVariableManager = new CSSVariableManager_1.default();
    connection.onInitialize(async (params) => {
        const capabilities = params.capabilities;
        // Does the client support the `workspace/configuration` request?
        // If not, we fall back using global settings.
        hasConfigurationCapability = !!(capabilities.workspace && !!capabilities.workspace.configuration);
        hasWorkspaceFolderCapability = !!(capabilities.workspace && !!capabilities.workspace.workspaceFolders);
        hasDiagnosticRelatedInformationCapability = !!(capabilities.textDocument &&
            capabilities.textDocument.publishDiagnostics &&
            capabilities.textDocument.publishDiagnostics.relatedInformation);
        const result = {
            capabilities: {
                textDocumentSync: node_1.TextDocumentSyncKind.Incremental,
                // Tell the client that this server supports code completion.
                completionProvider: {
                    resolveProvider: true,
                },
                definitionProvider: true,
                hoverProvider: true,
                colorProvider: true,
            },
        };
        if (hasWorkspaceFolderCapability) {
            result.capabilities.workspace = {
                workspaceFolders: {
                    supported: true,
                },
            };
        }
        return result;
    });
    connection.workspace.onDidChangeWorkspaceFolders((e) => {
        connection.console.info("fast");
        connection.console.info(e.added.toString());
    });
    connection.onInitialized(async () => {
        if (hasConfigurationCapability) {
            // Register for all configuration changes.
            connection.client.register(node_1.DidChangeConfigurationNotification.type, undefined);
        }
        if (hasWorkspaceFolderCapability) {
            connection.workspace.onDidChangeWorkspaceFolders((_event) => {
                connection.console.log("Workspace folder change event received.");
            });
        }
        const workspaceFolders = await connection.workspace.getWorkspaceFolders();
        const validFolders = workspaceFolders === null || workspaceFolders === void 0 ? void 0 : workspaceFolders.map((folder) => (0, protocol_1.uriToPath)(folder.uri) || "").filter((path) => !!path);
        const settings = await getDocumentSettings();
        // parse and sync variables
        cssVariableManager.parseAndSyncVariables(validFolders || [], {
            ...CSSVariableManager_1.defaultSettings,
            ...settings,
        });
    });
    let globalSettings = CSSVariableManager_1.defaultSettings;
    // Cache the settings of all open documents
    const documentSettings = new Map();
    function logger(location, content) {
        connection.console.info(`logger: ${location}`);
        connection.console.info(JSON.stringify(content));
    }
    connection.onDidCloseTextDocument((event) => {
        connection.console.info("onDidCloseTextDocument");
        connection.console.info(event.textDocument.uri);
    });
    function getDocumentSettings() {
        const resource = "all";
        if (!hasConfigurationCapability) {
            return Promise.resolve(globalSettings);
        }
        let result = documentSettings.get(resource);
        if (!result) {
            result = connection.workspace.getConfiguration("cssVariables");
            documentSettings.set(resource, result);
        }
        return result;
    }
    documents.onDidOpen(async ({ document }) => {
        documentSettings.clear();
        cssVariableManager.clearAllCache();
        const map = new Map();
        map.clear();
        const variables = cssVariableManager.getAll().entries;
        const workspaceFolders = await connection.workspace.getWorkspaceFolders();
        const validFolders = workspaceFolders === null || workspaceFolders === void 0 ? void 0 : workspaceFolders.map((folder) => (0, protocol_1.uriToPath)(folder.uri) || "").filter((path) => !!path);
        const settings = await getDocumentSettings();
        logger("onDidOpen", {
            settings,
            workspaceFolders,
            validFolders,
        });
        // parse and sync variables
        cssVariableManager.parseAndSyncVariables(validFolders || [], {
            ...globalSettings,
            ...settings,
        });
    });
    // Only keep settings for open documents
    documents.onDidClose((e) => {
        connection.console.log("Closed: " + e.document.uri);
        documentSettings.delete(e.document.uri);
    });
    connection.onDidChangeConfiguration(async (change) => {
        var _a;
        logger("asdf", { change });
        if (hasConfigurationCapability) {
            // Reset all cached document settings
            documentSettings.clear();
            cssVariableManager.clearAllCache();
            const validFolders = await connection.workspace
                .getWorkspaceFolders()
                .then((folders) => folders === null || folders === void 0 ? void 0 : folders.map((folder) => (0, protocol_1.uriToPath)(folder.uri) || "").filter((path) => !!path));
            const settings = await getDocumentSettings();
            // parse and sync variables
            cssVariableManager.parseAndSyncVariables(validFolders || [], {
                ...globalSettings,
                ...settings,
            });
        }
        else {
            globalSettings = ((((_a = change.settings) === null || _a === void 0 ? void 0 : _a.cssVariables) || CSSVariableManager_1.defaultSettings));
        }
    });
    connection.onDidChangeWatchedFiles(({ changes }) => {
        logger("onDidChangeWatchedFiles", { changes });
        // update cached variables
        changes.forEach((change) => {
            const filePath = (0, protocol_1.uriToPath)(change.uri);
            if (filePath) {
                // remove variables from cache
                if (change.type === node_1.FileChangeType.Deleted) {
                    cssVariableManager.clearAllCache();
                    // cssVariableManager.clearFileCache(filePath);
                }
                else {
                    const content = fs.readFileSync(filePath, "utf8");
                    cssVariableManager.parseCSSVariablesFromText({
                        content,
                        filePath,
                    });
                }
            }
        });
    });
    // This handler provides the initial list of the completion items.
    connection.onCompletion((_textDocumentPosition) => {
        const doc = documents.get(_textDocumentPosition.textDocument.uri);
        if (!doc) {
            return [];
        }
        const offset = doc.offsetAt(_textDocumentPosition.position);
        const currentWord = (0, getCurrentWord_1.getCurrentWord)(doc, offset);
        const isFunctionCall = (0, isInFunctionExpression_1.isInFunctionExpression)(currentWord);
        const items = [];
        const variableOptions = cssVariableManager.getAll();
        logger("onCompletion", { variableOptions, currentWord });
        variableOptions.forEach((variable) => {
            logger("onCompletion", variable);
            const varSymbol = variable.symbol;
            const insertText = isFunctionCall
                ? varSymbol.name
                : `var(${varSymbol.name})`;
            const completion = {
                label: varSymbol.name,
                detail: varSymbol.value,
                documentation: varSymbol.value,
                insertText,
                kind: (0, isColor_1.default)(varSymbol.value)
                    ? node_1.CompletionItemKind.Color
                    : node_1.CompletionItemKind.Variable,
            };
            if (isFunctionCall) {
                completion.detail = varSymbol.value;
            }
            items.push(completion);
        });
        return items;
    });
    // This handler resolves additional information for the item selected in
    // the completion list.
    connection.onCompletionResolve((item) => {
        return item;
    });
    connection.onDocumentColor((params) => {
        const document = documents.get(params.textDocument.uri);
        if (!document) {
            return [];
        }
        const colors = [];
        const text = document.getText();
        const matches = (0, findAll_1.findAll)(/var\((?<varName>--[a-z-0-9]+)/g, text);
        const globalStart = { line: 0, character: 0 };
        matches.map((match) => {
            const start = (0, indexToPosition_1.indexToPosition)(text, match.index + 4);
            const end = (0, indexToPosition_1.indexToPosition)(text, match.index + match[0].length);
            const cssVariable = cssVariableManager.getAll().get(match.groups.varName);
            if (cssVariable === null || cssVariable === void 0 ? void 0 : cssVariable.color) {
                const range = {
                    start: {
                        line: globalStart.line + start.line,
                        character: (end.line === 0 ? globalStart.character : 0) + start.character,
                    },
                    end: {
                        line: globalStart.line + end.line,
                        character: (end.line === 0 ? globalStart.character : 0) + end.character,
                    },
                };
                colors.push({
                    color: cssVariable.color,
                    range,
                });
            }
        });
        return colors;
    });
    connection.onHover(async (params) => {
        const doc = documents.get(params.textDocument.uri);
        if (!doc) {
            return null;
        }
        const offset = doc.offsetAt(params.position);
        const currentWord = (0, getCurrentWord_1.getCurrentWord)(doc, offset);
        logger("onHover", { offset, currentWord });
        if (!currentWord)
            return null;
        const nornalizedWord = currentWord.slice(1);
        const cssVariable = cssVariableManager.getAll().get(nornalizedWord);
        logger("onHover", { nornalizedWord, cssVariable });
        if (cssVariable) {
            return {
                contents: cssVariable.symbol.value,
                range: cssVariable.definition.range,
            };
        }
        return null;
    });
    connection.onColorPresentation((params) => {
        const document = documents.get(params.textDocument.uri);
        const className = document.getText(params.range);
        if (!className) {
            return [];
        }
        return [];
    });
    connection.onDefinition((params) => {
        const doc = documents.get(params.textDocument.uri);
        if (!doc) {
            return null;
        }
        const offset = doc.offsetAt(params.position);
        const currentWord = (0, getCurrentWord_1.getCurrentWord)(doc, offset);
        if (!currentWord)
            return null;
        const nornalizedWord = currentWord.slice(1);
        const cssVariable = cssVariableManager.getAll().get(nornalizedWord);
        const allObject = [...cssVariableManager.getAll().entries()].reduce((acc, [key, value]) => {
            acc[key] = value;
            return acc;
        }, {});
        connection.console.log(JSON.stringify(allObject));
        if (cssVariable) {
            return cssVariable.definition;
        }
        return null;
    });
    // Make the text document manager listen on the connection
    // for open, change and close text document events
    documents.listen(connection);
    // Listen on the connection
    return connection;
};
exports.makeConnection = makeConnection;
//# sourceMappingURL=connection.js.map